'use strict';

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _jade = require('jade');

var _jade2 = _interopRequireDefault(_jade);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _webpackUtils = require('./webpack-utils');

var _webpackUtils2 = _interopRequireDefault(_webpackUtils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var read = _ramda2.default.partialRight(_fs.readFileSync, ['utf8']);

var path = _ramda2.default.partial(_path2.default.join, [__dirname]);

path.template = _ramda2.default.partial(path, ['../templates']);

var defaultConfig = {
  meta: {
    charset: 'UTF-8',
    viewport: 'width=device-width, \
      initial-scale=1, \
      minimum-scale=1.0, \
      maximum-scale=1.0, \
      user-scalable=no',
    'apple-mobile-web-app-capable': 'yes',
    'mobile-web-app-capable': 'yes'
  },
  compile: {
    pretty: true,
    doctype: 'html'
  }
};

var getTemplate = function getTemplate() {
  var customTemplatePath = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];

  var filePath = undefined,
      contents = undefined;

  if (customTemplatePath) {
    /* Our API is designed such that the custom template must
    extend a base layout. To achieve this we use Jade's `extend`
    inheritance feature which requires a relative file path from source
    file location to the parent being extended. */

    var parentTemplatePath = _path2.default.relative(_path2.default.dirname(customTemplatePath), path.template('default.jade'));
    filePath = customTemplatePath;
    contents = 'extends ' + parentTemplatePath + '\n\n' + read(customTemplatePath);
  } else {
    filePath = path.template('./default.jade');
    contents = read(filePath);
  }

  return [filePath, contents];
};

var createRenderer = function createRenderer(pluginJadeOptions) {
  var templateExtension = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  /* Note regarding Base Settings. It allow the user to set options
  documented in the Jade docs here http://jade-lang.com/api.
  The filename setting is required but actually an implementation
  detail from _our_ user's point of view. Therefore we combine the
  optional high-level options with this automatically resolved
  low-level option. */

  var _getTemplate = getTemplate(templateExtension);

  var _getTemplate2 = _slicedToArray(_getTemplate, 2);

  var filename = _getTemplate2[0];
  var source = _getTemplate2[1];

  var jadeOptions = (0, _lodash2.default)({}, { filename: filename }, pluginJadeOptions);
  var render = _jade2.default.compile(source, jadeOptions);
  return render;
};

var templateSettingsBase = {
  attributes: {
    html: {},
    head: {},
    body: {}
  },
  headers: {
    mobile: true,
    utf8: true
  },
  react: {
    mainGlobalIdentifier: 'Main',
    autoRender: true
    // + Dynamically Resolved Fields.
  },
  mainElement: {
    id: 'main'
  }
};

var entryToTemplateSetting = _ramda2.default.curry(function (overrides, _ref) {
  var _ref2 = _slicedToArray(_ref, 2);

  var name = _ref2[0];
  var assetPathsByType = _ref2[1];

  var settings = (0, _lodash2.default)({}, templateSettingsBase, overrides, { assetPathsByType: assetPathsByType, name: name });

  return Object.assign(settings, {
    mainQuerySelector: settings.mainElement ? '#' + settings.mainElement.id : 'body'
  });
});

var create = function create() {
  var customConfig = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var config = (0, _lodash2.default)({}, defaultConfig, customConfig);

  var apply = function apply(compiler) {
    compiler.plugin('emit', function (c, cb) {
      var entries = undefined;
      var stats = c.getStats().toJson();

      try {
        /* stats.assetsByChunkName is a mapping of entries to chunks.
        For example if User had configured:
         `{ entry: { foo: 'bar' }, ...}`
         then the mapping would at least contain:
         `{ foo: ['bar'] }`.
        */
        entries = _ramda2.default.mapObj(_webpackUtils2.default.getAssetsByEntry(_webpackUtils2.default.getPublicPath(c), stats.hash), stats.assetsByChunkName);
      } catch (e) {
        throw new Error('Failed to process public assets: ' + e.message);
      }

      /* If using hot-enabled webpack-dev-server then add its necessary runtime
      script to jsAssets so that it is automatically included in
      our final html. */

      if (_webpackUtils2.default.isHotMode(c)) _ramda2.default.mapObj(entries, _webpackUtils2.default.addChunkDevServer);

      /* Generate the Jade rendering function. */

      var render = createRenderer(config.compile, config.templateExtension);

      /* Generate the HTML */

      var isSingleEntry = _ramda2.default.equals(1, _ramda2.default.length(_ramda2.default.keys(entries)));

      _ramda2.default.pipe(_ramda2.default.toPairs, _ramda2.default.map(entryToTemplateSetting(config)), _ramda2.default.forEach(function (templateSettings) {
        /* If there is only one entry we use the special name `index` which
        browsers will automatically load when no file is given in the URI. */
        var fileName = isSingleEntry ? 'index.html' : templateSettings.name + '.html';
        _webpackUtils2.default.addAssetHTML(c.assets, fileName, render(templateSettings));
      }))(entries);

      cb();
    });
  };

  return {
    apply: apply
  };
};

exports.default = create;